package com.baidu.rtc.sample.util;


import android.graphics.Color;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.baidu.rtc.sample.data.ContextConstant;

public class CToast {

    private static Toast mToast;

    private static int mLayoutId = -1;
    private static int mTextId = -1;

    private static boolean mIsGame = false;

    public static void showToastMessage(int resId) {
        showToastMessage(resId, Toast.LENGTH_SHORT);
    }

    public static void showToastMessage(int resId, int duration) {
        showToastMessage(ContextConstant.getContext().getResources().getString(resId), duration);
    }

    public static void showToastMessage(String text) {
        showToastMessage(text, Toast.LENGTH_SHORT);
    }

    public static void showToastMessage(String text, int duration) {
        showToastWithSpannableText(text, duration, null);
    }


    public static void showToastMessage(String text, int duration, int gravity) {
        showToastWithSpannableText(text, duration, null, gravity);
    }

    public static void showToastMessage(int resId, int duration, int gravity) {
        showToastWithSpannableText(
                ContextConstant.getContext().getResources().getString(resId),
                duration, null, gravity);
    }

    public static void showToastWithSpannableText(String text,
                                                  int duration, String spannableText) {
        showToastWithSpannableText(text, duration, spannableText, -1);
    }

    public static void showToastWithSpannableText(String text,
                                                  int duration, String spannableText, int gravity) {
        if (mLayoutId == -1 || mTextId == -1) {
            if (mToast == null) {
                mToast = Toast.makeText(ContextConstant.getContext(), text, duration);
                if (gravity > -1) {
                    mToast.setGravity(gravity, 0, 0);
                }
            } else {
                mToast.cancel();
                mToast = Toast.makeText(ContextConstant.getContext(), text, duration);
                if (gravity > -1) {
                    mToast.setGravity(gravity, 0, 0);
                }
                mToast.setDuration(duration);
            }
            try {
                mToast.show();
            } catch (Exception e) {
                // nothing
            }
        } else {
            if (mToast != null) {
                mToast.cancel();
            }

            mToast = new Toast(ContextConstant.getContext());
            if (gravity > -1) {
                mToast.setGravity(gravity, 0, 0);
            }
            mToast.setDuration(duration);
            LayoutInflater mInflater = LayoutInflater.from(ContextConstant.getContext());
            try {
                View v = mInflater.inflate(mLayoutId, null);
                TextView textView = (TextView) v.findViewById(mTextId);
                if (!TextUtils.isEmpty(spannableText)
                        && !TextUtils.isEmpty(text)
                        && text.contains(spannableText)) {
                    SpannableStringBuilder style = new SpannableStringBuilder(text);
                    ForegroundColorSpan span =
                            new ForegroundColorSpan(Color.parseColor("#ffca00"));
                    int start = text.indexOf(spannableText);
                    int end = text.indexOf(spannableText) + spannableText.length();
                    style.setSpan(span, start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
                    textView.setText(style);
                } else {
                    textView.setText(text);
                }
                mToast.setView(v);
                mToast.show();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static void setCustomToastRes(int layoutId, int textId) {
        mLayoutId = layoutId;
        mTextId = textId;
    }

    public static void setGameFlag(boolean isGame) {
        mIsGame = isGame;
    }

}
